SystemOrganization addCategory: #'AJP-Core'!

WAServerAdaptor subclass: #AJPAdaptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPAdaptor class methodsFor: 'testing' stamp: 'pmm 1/6/2009 20:47'!
isAbstract
	^self name = #AJPAdaptor! !

!AJPAdaptor methodsFor: 'defaults' stamp: 'pmm 7/17/2010 15:14'!
defaultCodec
	^ GRCodec forEncoding: 'utf-8'! !

!AJPAdaptor methodsFor: 'testing' stamp: 'pmm 7/5/2009 13:03'!
isUrlEncoded: anAjpRequest
	"answers whether the body of the given request is url encoded"
	^anAjpRequest contentType = 'application/x-www-form-urlencoded'! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 8/27/2009 10:03'!
mimeParserClass
	self subclassResponsibility! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 8/27/2009 11:06'!
mimePartsOf: anAjpRequest do: aOneArgumentBlock
	(self mimeParserClass new
		parse: anAjpRequest body
		boundary: anAjpRequest multiPartBoundary) do: aOneArgumentBlock! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 12/27/2008 16:54'!
newWriteStream
	^WriteStream on: (String new: 64)! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 11/7/2010 14:02'!
postFieldsOf: anAjpRequest into: aRequestFields
	(self isUrlEncoded: anAjpRequest)
		ifTrue: [
			self
				queryString: (self requestBodyFor: anAjpRequest)
				into: aRequestFields ]
		ifFalse: [
			self mimePartsOf: anAjpRequest do: [ :each |
				| fieldName fileName data |
				fieldName := each fieldName.
				fileName := each fileName.
				data := fileName isEmptyOrNil
					ifTrue: [
						self codec url decode: each body ]
					ifFalse: [
						| contentType |
						contentType := each contentType.
						WAFile new
							fileName: (self codec url decode: fileName);
							"can be missing on Chrome"
							contentType: (contentType isNil ifFalse: [ contentType seasideMimeType ]);
							contents: each body;
							yourself ].
				aRequestFields at: fieldName add: data ] ]! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 7/17/2010 17:38'!
putKey: aKeyStream value: aValueStream into: aRequestFields
	| value |
	value := aValueStream contents.
	aRequestFields
		at: aKeyStream contents
		add: (value isEmptyOrNil
			ifTrue: [ nil ]
			ifFalse: [ self codec url decode: (WAUrl decodePercent: value) ])! !

!AJPAdaptor methodsFor: 'private' stamp: 'pmm 7/17/2010 17:37'!
queryString: aString into: aRequestFields
	| stream keyStream valueStream readKey |
	aString isNil ifFalse: [
		stream := aString readStream.
		readKey := true.
		keyStream := self newWriteStream.
		valueStream := self newWriteStream.
		[ stream atEnd ] whileFalse: [
			| next |
			next := stream next.
			next = $=
				ifTrue: [ readKey := false ]
				ifFalse: [ next = $&
					ifTrue: [
						self putKey: keyStream value: valueStream into: aRequestFields.
						readKey := true.
						keyStream := self newWriteStream.
						valueStream := self newWriteStream ]
					ifFalse: [ (readKey ifTrue: [ keyStream ] ifFalse: [ valueStream ]) nextPut: next ] ] ].
		self putKey: keyStream value: valueStream into: aRequestFields ]! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/4/2009 08:49'!
requestAddressFor: anAjpRequest
	^anAjpRequest remoteHost! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 11/7/2010 14:02'!
requestBodyFor: anAjpRequest
	^ anAjpRequest body! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/4/2009 08:50'!
requestCookiesFor: anAjpRequest
	^ Array streamContents: [:stream |
		anAjpRequest cookieHeaders do: [ :each |
			stream nextPutAll: (WARequestCookie fromString: each)] ]! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/17/2010 15:16'!
requestFieldsFor: anAjpRequest
	| fields |
	fields := WARequestFields new.
	anAjpRequest isPost
		ifTrue: [ self postFieldsOf: anAjpRequest into: fields ].
	^ fields! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/4/2009 08:56'!
requestHeadersFor: anAjpRequest
	| headers fields |
	headers := anAjpRequest headers.
	fields := Dictionary new: headers size.
	headers do: [ :each |
		fields at: each key asLowercase put: each value ].
	^fields! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/4/2009 08:57'!
requestMethodFor: anAjpRequest
	^anAjpRequest method! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 7/17/2010 17:33'!
requestUrlFor: anAjpRequest
	| url queryString urlString |
	queryString := anAjpRequest queryString.
	urlString := queryString isEmptyOrNil
		ifTrue: [ anAjpRequest requestUri ]
		ifFalse: [ anAjpRequest requestUri, '?', queryString ].
	url := (WAUrl absolute: urlString)
		decodedWith: self codec.
	anAjpRequest isSsl
		ifTrue: [ url scheme: 'https' ].
	^ url! !

!AJPAdaptor methodsFor: 'converting-request' stamp: 'pmm 8/3/2009 19:58'!
requestVersionFor: anAjpRequest
	^ WAHttpVersion readFrom: anAjpRequest protocol readStream! !

!AJPAdaptor methodsFor: 'converting' stamp: 'pmm 7/17/2010 15:33'!
responseFrom: aRequestContext
	| partialHeaders cookies fullHeaders seasideResponse |
	
	seasideResponse := aRequestContext response.
	partialHeaders := seasideResponse headers.
	cookies := seasideResponse cookies.
	fullHeaders := OrderedCollection new: partialHeaders size + (cookies size * 2) + 3. "content type + content-length, servlet engine".
	partialHeaders keysAndValuesDo: [ :key :value |
		fullHeaders add: key -> value greaseString ].
	cookies do: [ :each |
		fullHeaders add: 'Set-Cookie' -> each oldNetscapeString.
		fullHeaders add: 'Set-Cookie2' -> each rfc2965String ].
	fullHeaders add: 'Content-Type' -> seasideResponse contentType greaseString.
	^AJPResponse
		status: seasideResponse status
		headers: fullHeaders
		body: seasideResponse contents! !

GRObject subclass: #AJPBodyChunk
	instanceVariableNames: 'start size body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPBodyChunk class methodsFor: 'instance creation' stamp: 'pmm 10/31/2010 10:57'!
start: aStartInteger size: aSizeInteger body: aStringOrByteArray
	^ self basicNew
		initializeWithStart: aStartInteger
		size: aSizeInteger
		body: aStringOrByteArray! !

!AJPBodyChunk methodsFor: 'accessing' stamp: 'pmm 10/31/2010 10:53'!
body
	^ body! !

!AJPBodyChunk methodsFor: 'accessing' stamp: 'pmm 10/31/2010 10:54'!
end
	^ start + size - 1! !

!AJPBodyChunk methodsFor: 'initialize-release' stamp: 'pmm 10/31/2010 10:57'!
initializeWithStart: aStartInteger size: aSizeInteger body: aStringOrByteArray
	start := aStartInteger.
	size := aSizeInteger.
	body := aStringOrByteArray! !

!AJPBodyChunk methodsFor: 'accessing' stamp: 'pmm 10/31/2010 10:54'!
size
	^ size! !

!AJPBodyChunk methodsFor: 'accessing' stamp: 'pmm 10/31/2010 10:54'!
start
	^ start! !

GRObject subclass: #AJPDataStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPDataStream commentStamp: 'pmm 7/19/2008 11:58' prior: 0!
An AJPDataStream is a helper class that can read AJP data packets.
!

!AJPDataStream methodsFor: 'public' stamp: 'pmm 12/8/2008 23:14'!
destroy
	"releases all native resources like socket handles"! !

!AJPDataStream methodsFor: 'public' stamp: 'pmm 8/24/2008 17:08'!
flush
	"writes all data from the buffers to the socket"! !

!AJPDataStream methodsFor: 'hinting' stamp: 'pmm 10/30/2010 10:40'!
hintNewBinaryPacket
	"hint to the implementation that the next packet will be binary"! !

!AJPDataStream methodsFor: 'hinting' stamp: 'pmm 10/30/2010 10:40'!
hintNewTextPacket
	"hint to the implementation that the next packet will be text"! !

!AJPDataStream methodsFor: 'private' stamp: 'pmm 1/10/2009 13:44'!
protocolError: aString
	AJPProtocolError signal: aString! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 7/19/2008 12:02'!
readBoolean
	"returns the next boolean
	
	Spec:
	A single byte, 1 = true, 0 = false. Using other non-zero values as true (i.e. C-style) may work in some places, but it won't in others."
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 7/19/2008 12:02'!
readByte
	"returns the next byte
	
	Spec:
	A single byte."
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 17:18'!
readChunk: anInteger into: aCollection startingAt: startIndex
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 7/19/2008 12:03'!
readInteger
	"returns the next integer
	
	Spec:
	A number in the range of 0 to 2^16 (32768). Stored in 2 bytes with the high-order byte first."
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 7/19/2008 12:03'!
readString
	"returns the next String
	
	Spec:
	A variable-sized string (length bounded by 2^16). Encoded with the length packed into two bytes first, followed by the string (including the terminating '\0'). Note that the encoded length does not include the trailing '\0' -- it is like strlen. This is a touch confusing on the Java side, which is littered with odd autoincrement statements to skip over these terminators. I believe the reason this was done was to allow the C code to be extra efficient when reading strings which the servlet container is sending back -- with the terminating \0 character, the C code can pass around references into a single buffer, without copying. If the \0 was missing, the C code would have to copy things out in order to get its notion of a string. Note a size of -1 (65535) indicates a null string and no data follow the length in this case."
	
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'reading' stamp: 'pmm 7/20/2008 10:17'!
readStringWithSizeHighByte: aByte
	"reads a String with the high byte of the size already read
	otherwise the same as #readString"! !

!AJPDataStream methodsFor: 'hinting' stamp: 'pmm 8/24/2008 16:34'!
sizeHint: anInteger
	"hint to the implementation that anInteger data could be read in one go"! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 7/20/2008 15:29'!
writeBoolean: aBoolean
	"writes a single boolean
	see #readBoolean"
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 7/20/2008 15:29'!
writeByte: aByte
	"writes a single byte
	see #readByte"
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 11/23/2008 14:14'!
writeBytes: aByteArray
	"writes an array of bytes"
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 10/31/2010 11:00'!
writeChunk: anInteger from: aCollection startingAt: startIndex
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 7/20/2008 15:29'!
writeInteger: anInteger
	"writes a two byte integer
	see #readInteger"
	self subclassResponsibility! !

!AJPDataStream methodsFor: 'writing' stamp: 'pmm 7/20/2008 15:29'!
writeString: aString
	"writes a String
	see #readString"
	self subclassResponsibility! !

AJPDataStream subclass: #AJPDefaultDataStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPDefaultDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 17:12'!
readBoolean
	"returns the next boolean
	
	Spec:
	A single byte, 1 = true, 0 = false. Using other non-zero values as true (i.e. C-style) may work in some places, but it won't in others."
	^self readByte ~= 0! !

!AJPDefaultDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 17:15'!
readInteger
	"returns the next integer
	
	Spec:
	A number in the range of 0 to 2^16 (32768). Stored in 2 bytes with the high-order byte first."
	^(self readByte << 8) bitOr: (self readByte)! !

!AJPDefaultDataStream methodsFor: 'reading' stamp: 'pmm 10/30/2010 17:16'!
readString
	"returns the next String
	
	Spec:
	A variable-sized string (length bounded by 2^16). Encoded with the length packed into two bytes first, followed by the string (including the terminating '\0'). Note that the encoded length does not include the trailing '\0' -- it is like strlen. This is a touch confusing on the Java side, which is littered with odd autoincrement statements to skip over these terminators. I believe the reason this was done was to allow the C code to be extra efficient when reading strings which the servlet container is sending back -- with the terminating \0 character, the C code can pass around references into a single buffer, without copying. If the \0 was missing, the C code would have to copy things out in order to get its notion of a string. Note a size of -1 (65535) indicates a null string and no data follow the length in this case."
	
	^self readStringWithSizeHighByte: self readByte! !

!AJPDefaultDataStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 17:10'!
writeBoolean: aBoolean
	self writeByte: (aBoolean ifTrue: [ 1 ] ifFalse: [ 0 ])! !

!AJPDefaultDataStream methodsFor: 'writing' stamp: 'pmm 10/30/2010 17:10'!
writeInteger: anInteger
	self writeByte: (anInteger bitAnd: 16rFF00) >> 8.
	self writeByte: (anInteger bitAnd: 16rFF)! !

GRObject subclass: #AJPLoop
	instanceVariableNames: 'stream shouldStop server'
	classVariableNames: 'CPing ForwardRequest Ping Shutdown'
	poolDictionaries: ''
	category: 'AJP-Core'!

AJPLoop subclass: #AJPAdapterLoop
	instanceVariableNames: 'adapter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPAdapterLoop class methodsFor: 'instance creation' stamp: 'pmm 10/7/2010 13:38'!
on: aSocket server: aServer adapter: aServerAdapter
	^self new
		initializeOn: aSocket server: aServer;
		adapter: aServerAdapter;
		yourself! !

!AJPAdapterLoop methodsFor: 'initialize-release' stamp: 'pmm 12/28/2008 17:00'!
adapter: aServerAdapter
	adapter := aServerAdapter! !

!AJPAdapterLoop methodsFor: 'actions' stamp: 'pmm 8/31/2009 16:23'!
forwardRequest: anAjpRequest
	^[ adapter process: anAjpRequest ]
		on: Error
		do: [ :error |
			AJPResponse
				status: 500
				headers: OrderedCollection new
				body: '' ]! !

!AJPLoop class methodsFor: 'instance creation' stamp: 'pmm 10/7/2010 13:38'!
on: anAjpDataStream server: aServer
	^self new
		initializeOn: anAjpDataStream server: aServer;
		yourself! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 11/23/2008 13:41'!
cping
	"The web server asks the container to respond quickly with a CPong."
	self writePreamble.
	stream writeInteger: 1. "packet size"
	"9  	CPong Reply  	The reply to a CPing request"
	stream writeByte: 16r9.
	stream flush! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 8/24/2008 17:04'!
cpingCode
	"The web server asks the container to respond quickly with a CPong."
	^10! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 8/29/2009 10:13'!
destroy
	stream destroy.
	shouldStop := true! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 8/24/2008 17:03'!
forwardCode
	"Begin the request-processing cycle with the following data."
	^2! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 11/23/2008 14:51'!
forwardRequest: anAjpRequest
	"processes the given AJPRequest instance
	answers an AJPResponse"
	self subclassResponsibility! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 10/31/2010 16:53'!
getBodyChunk: anInteger
	"Get further data from the request if it hasn't all been transferred yet.
	Answer the packet size in bytes.
	
	According to spec if there is no more data the server will send an empty packet (size 0). However in practice Apache 2.2 sends a packet of size 2, the two bytes mark an empty string."
	self writePreamble.
	stream writeInteger: 3. "packet size code + size"
	"6 	Get Body Chunk  	Get further data from the request if it hasn't all been transferred yet"
	stream writeByte: 16r6.
	stream writeInteger: anInteger.
	stream flush.
	stream hintNewBinaryPacket.
	^self readPreamble! !

!AJPLoop methodsFor: 'initialize-release' stamp: 'pmm 10/7/2010 13:36'!
initializeOn: aStream server: aServer
	stream := aStream.
	server := aServer.
	shouldStop := false! !

!AJPLoop methodsFor: 'reading' stamp: 'pmm 10/7/2010 13:41'!
loop
	[ 
		[ [ [ self loopPossiblyThrowingExceptions ]
			on: ConnectionClosed
			do: [ :error |
				stream destroy.
				server removeWorker: self.
				^self ] ]
			on: AJPLoopTerminatedError
			do: [ :error |
				server removeWorker: self.
				^self ] ]
			on: Error
			do: [ :error |
				stream destroy.
				server removeWorker: self.
				^self ]
	 ] repeat! !

!AJPLoop methodsFor: 'reading' stamp: 'pmm 10/30/2010 09:54'!
loopPossiblyThrowingExceptions
	| type length |
	stream hintNewTextPacket..
	length := self readPreamble.
	length = 2 ifTrue: [
		"srsly WTF"
		stream readByte.
		stream readByte.
		^self ].
	type := stream readByte.
	
	type = self forwardCode ifTrue: [
		| request response |
		request := self readRequest.
		response := self forwardRequest: request.
		self writeResponse: response.
		^self ].
		
	type = self shutdownCode ifTrue: [
		self shutdown.
		^self ].
	
	type = self cpingCode ifTrue: [
		self cping.
		^self ].
		
	type = self pingCode ifTrue: [
		self ping.
		^self ]! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 10/31/2010 16:59'!
maximumBodySize
	^ 16r2000 - 6! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 8/24/2008 16:52'!
ping
	"The web server asks the container to take control (secure login phase)."
	self notYetImplemented! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 8/24/2008 17:04'!
pingCode
	"The web server asks the container to take control (secure login phase)."
	^8! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 1/10/2009 13:44'!
protocolError: aString
	AJPProtocolError signal: aString! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 12/8/2008 19:51'!
readIgnoringTimeout
	"reads a byte from the data stream ignoring timeouts
	relationale:
		- as long as the no browser connects Apache can send no data
		- we must check for shutdown signals"
	self subclassResponsibility! !

!AJPLoop methodsFor: 'reading' stamp: 'pmm 1/3/2009 12:31'!
readPreamble
	"reads the preamble from the package"
	| dataLength |
	"0x12  	0x34 Data Length
	type"
	self readIgnoringTimeout = 16r12
		ifFalse: [ self protocolError: 'expected 0x12' ].
	stream readByte = 16r34
		ifFalse: [ self protocolError: 'expected 0x34' ].
		
	dataLength := stream readInteger.
	stream sizeHint: dataLength.
	^dataLength! !

!AJPLoop methodsFor: 'reading' stamp: 'pmm 10/31/2010 17:03'!
readRequest
	| request reader |
	reader := AJPRequestReader on: stream.
	request := reader read.
	reader hasContentLength ifTrue: [
		request body: (self readRequestBody: request) ].
	^request! !

!AJPLoop methodsFor: 'reading' stamp: 'pmm 11/7/2010 11:54'!
readRequestBody: aRequest
	| body contentLength read |
	contentLength := aRequest contentLength.
	body := String new: contentLength.
	read := 0.
	[ read < contentLength ] whileTrue: [
		| chunkSize packetSize toRead left |
		left := contentLength - read.
		toRead := left min: self maximumBodySize.
		packetSize := self getBodyChunk: toRead.
		packetSize > 0
			ifTrue: [
				chunkSize := stream readInteger.
				chunkSize > 0
					ifTrue: [
						stream readChunk: chunkSize into: body startingAt: read + 1.
						read := read + chunkSize ] ]
			ifFalse: [
				"premature end shouldn't happen"
				^ body copyFrom: 1 to: read ] ].
	^ body! !

!AJPLoop methodsFor: 'actions' stamp: 'pmm 8/24/2008 16:51'!
shutdown
	"The web server asks the container to shut itself down."
	self notYetImplemented! !

!AJPLoop methodsFor: 'private' stamp: 'pmm 8/24/2008 17:04'!
shutdownCode
	"The web server asks the container to shut itself down."
	^7! !

!AJPLoop methodsFor: 'writing' stamp: 'pmm 8/24/2008 17:11'!
writePreamble
	"writes the preamble to the package"
	stream writeByte: 65. "$A asciiValue"
	stream writeByte: 66 "$B asciiValue"! !

!AJPLoop methodsFor: 'writing' stamp: 'pmm 12/7/2010 21:10'!
writeResponse: anAjpResponse
	| writer |
	writer := AJPResponseWriter on: stream response: anAjpResponse.
	self writePreamble.
	writer sendHeaders.
	stream flush.
	
	1 to: writer numberOfBodyChunks do: [ :index |
		self writePreamble.
		writer sendBodyChunk: index.
		stream flush ].
	
	self writePreamble.
	writer sendEndResponse: true.
	stream flush! !

GRObject subclass: #AJPMimeParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 11:34'!
copy: aSourceByteArray into: aTargetByteArray startingAt: sourcePosition
	self subclassresponsibility! !

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 14:11'!
copy: aSourceByteArray into: aTargetByteArray startingAt: sourceStart upTo: sourceEnd
	self subclassresponsibility! !

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 16:39'!
indexOfSubCollection: bondary in: aString startingAt: start
	| next |
	next := aString
		indexOfSubCollection: bondary
		startingAt: start + 2
		ifAbsent: [ ^ aString size ].
	((aString at: next - 1) = $-
		and: [ (aString at: next - 2) = $- ]) ifTrue: [ ^ next ].
	"should only really rarely happen"
	^ self indexOfSubCollection: bondary in: aString startingAt: start! !

!AJPMimeParser methodsFor: 'public' stamp: 'pmm 11/7/2010 13:54'!
parse: aContentString boundary: aBoundaryString
	| parts |
	parts := OrderedCollection new.
	self parse: aContentString boundary: aBoundaryString do: [ :start :end |
		parts add: (self parseChunk: aContentString startingAt: start upTo: end) ].
	^parts! !

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 16:40'!
parse: aContentString boundary: aBoundaryString do: aTwoArgumentBlock
	| next parts start |
	start := aBoundaryString size + 1 + 2 "we don't look for --".
	next := (self indexOfSubCollection: aBoundaryString in: aContentString startingAt: start) - 2.
	[ next < (aContentString size - 2) ] whileTrue: [
		aTwoArgumentBlock value: start + 2 value: next - 1.
		start := next + aBoundaryString size + 2.
		next := (self indexOfSubCollection: aBoundaryString in: aContentString startingAt: start) - 2 ]! !

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 14:12'!
parseChunk: aContentString startingAt: initialStart upTo: end
	| crlf start next headers atEnd part |
	crlf := String crlf.
	headers := WAHeaderFields new.
	start := initialStart.
	atEnd := false.
	next := aContentString
		indexOfSubCollection: crlf
		startingAt: start 
		ifAbsent: [ 0 ].
	[ atEnd ] whileFalse: [
		self parseHeader: aContentString startingAt: start upTo: next - 1 into: headers.
		start := next + 2.
		atEnd := (aContentString at: next + 2) = Character cr
			and: [ (aContentString at: next + 3) = Character lf ].
		atEnd ifFalse: [
			next := aContentString
				indexOfSubCollection: crlf
				startingAt: start 
				ifAbsent: [ 0 ] ] ].
	part := AJPMimePart headers: headers.
	part body: (part fileName isNil
		ifTrue: [
			"use a String for non file bodies (saves copy for field decoding)"
			aContentString copyFrom: next + 4 to: end - 2 ]
		ifFalse: [
			"use a byte array for file bodies (saves copy for WAFile)"
			| size content |
			size := (end - 2) - (next + 4) + 1.
			content := ByteArray new: size.
			self copy: aContentString into: content startingAt: next + 4 upTo: end - 2.
			content ]).
	^part! !

!AJPMimeParser methodsFor: 'private' stamp: 'pmm 11/7/2010 13:45'!
parseHeader: aString startingAt: start upTo: end into: aHeaderFields
	| dotIndex key value |
	dotIndex := aString indexOf: $: startingAt: start + 1.
	key := aString copyFrom: start to: dotIndex - 1.
	value := (aString copyFrom: dotIndex + 1 to: end) trimBoth.
	aHeaderFields at: key add: value! !

GRObject subclass: #AJPMimePart
	instanceVariableNames: 'headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPMimePart class methodsFor: 'instance creation' stamp: 'pmm 11/7/2010 14:05'!
headers: aHeaderFields
	^self basicNew initializeWithHeaders: aHeaderFields! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 15:29'!
body
	^body! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 11/7/2010 14:06'!
body: aStringOrByteArray
	body :=  aStringOrByteArray! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 21:32'!
contentDisposition
	^headers at: 'Content-Disposition'! !

!AJPMimePart methodsFor: 'private' stamp: 'pmm 8/26/2009 21:52'!
contentDispositionValues
	| contentDisposition values start semiColonIndex |
	contentDisposition := self contentDisposition.
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentDisposition copyFrom: start to: semiColonIndex - 1.
		values add: value trimBoth.
		start := semiColonIndex + 1.
		semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (contentDisposition copyFrom: start to: contentDisposition size) trimBoth.
	^values! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 15:56'!
contentType
	^headers at: 'Content-Type'! !

!AJPMimePart methodsFor: 'private' stamp: 'pmm 8/26/2009 21:53'!
detectContentDispositionValue: aString
	self contentDispositionValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				| value |
				value := each copyAfter: $=.
				(value first = $" and: [ value last = $" ])
					ifTrue: [ ^value copyFrom: 2 to: value size -1 ]
					ifFalse: [ ^value ] ] ].
	^nil! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 21:41'!
fieldName
	^self detectContentDispositionValue: 'name'! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 21:41'!
fileName
	^self detectContentDispositionValue: 'filename'! !

!AJPMimePart methodsFor: 'testing' stamp: 'pmm 7/18/2010 15:42'!
hasBody
	^ body notNil! !

!AJPMimePart methodsFor: 'accessing' stamp: 'pmm 8/26/2009 15:29'!
headers
	^headers! !

!AJPMimePart methodsFor: 'initialize-release' stamp: 'pmm 11/7/2010 14:05'!
initializeWithHeaders: aHeaderFields
	super initialize.
	headers := aHeaderFields! !

GRObject subclass: #AJPRequest
	instanceVariableNames: 'method protocol requestUri remoteAddress remoteHost serverName serverPort isSsl headers attributes body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPRequest commentStamp: 'pmm 7/13/2008 14:45' prior: 0!
An AJPRequest is xxxxxxxxx.

Instance Variables
	attributes:		<Object>
	headers:		<Object>
	isSsl:		<Object>
	method:		<String>
	protocol:		<Object>
	remoteAddress:		<Object>
	remoteHost:		<Object>
	requestUri:		<Object>
	serverName:		<Object>
	serverPort:		<Object>

attributes
	- xxxxx

headers
	- xxxxx

isSsl
	- xxxxx

method
	- the HTTP or WebDAV method name in all caps. 'UNKNOWN' if the method is unknown.

protocol
	- xxxxx

remoteAddress
	- xxxxx

remoteHost
	- xxxxx

requestUri
	- xxxxx

serverName
	- xxxxx

serverPort
	- xxxxx
!

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 7/13/2008 14:48'!
attributes
	^ attributes! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 7/13/2008 14:48'!
attributes: anObject
	attributes := anObject! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:10'!
authType
	"Answer the authentication type or nil if none is present.
	e.g. 'Basic' or 'Digest'"
	^self detectAttribute: 'auth_type'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 12/10/2008 07:35'!
body
	^ body! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 12/10/2008 07:35'!
body: anObject
	body := anObject! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 10/16/2010 16:11'!
contentLength
	"Answer the content length as an integer or nil."
	| value |
	value := self detectHeader: 'content-length'.
	^value isNil
		ifFalse: [ ^value greaseInteger ]! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 10/16/2010 16:12'!
contentType
	"Answer the content length as an integer or nil."
	^self detectHeader: 'content-type'! !

!AJPRequest methodsFor: 'private' stamp: 'pmm 8/27/2009 10:14'!
contentTypeValues
	| contentType values start semiColonIndex |
	contentType := self contentType.
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentType indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentType copyFrom: start to: semiColonIndex - 1.
		values add: value trimBoth.
		start := semiColonIndex + 1.
		semiColonIndex := contentType indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (contentType copyFrom: start to: contentType size) trimBoth.
	^values! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 12/27/2008 21:57'!
cookieHeaders
	^ (self headers
		select: [ :each | each key = 'cookie' ])
		collect: [ :each | each value ]! !

!AJPRequest methodsFor: 'private' stamp: 'pmm 10/16/2010 15:49'!
detectAttribute: aString
	"answers the matching attribute or nil"
	attributes do: [ :each |
		each key = aString
			ifTrue: [ ^each value ] ].
	^nil! !

!AJPRequest methodsFor: 'private' stamp: 'pmm 8/27/2009 10:15'!
detectContentTypeValue: aString
	self contentTypeValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				| value |
				value := each copyAfter: $=.
				(value first = $" and: [ value last = $" ])
					ifTrue: [ ^value copyFrom: 2 to: value size -1 ]
					ifFalse: [ ^value ] ] ].
	^nil! !

!AJPRequest methodsFor: 'private' stamp: 'pmm 12/27/2008 22:00'!
detectHeader: aString
	"answers the matching header or nil"
	headers do: [ :each |
		each name = aString
			ifTrue: [ ^each value ] ].
	^nil! !

!AJPRequest methodsFor: 'testing' stamp: 'pmm 7/17/2010 15:32'!
hasBody
	^ body notNil! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 7/13/2008 14:48'!
headers
	^ headers! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 12/8/2008 23:34'!
headers: aCollectionOfAssociations
	headers := aCollectionOfAssociations! !

!AJPRequest methodsFor: 'testing' stamp: 'pmm 7/4/2009 08:55'!
isPost
	^self method = 'POST'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
isSsl
	^ isSsl! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
isSsl: aBoolean
	isSsl := aBoolean! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:10'!
jvmRoute
	"Answers the jvmRoute or nil if none is present."
	^self detectAttribute: 'jvm_route'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:42'!
method
	^ method! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:42'!
method: aString
	method := aString! !

!AJPRequest methodsFor: 'accessing-headers' stamp: 'pmm 10/16/2010 16:12'!
multiPartBoundary
	"Answer the multipart boundary or nil if there's none."
	^self detectContentTypeValue: 'boundary'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
protocol
	^ protocol! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
protocol: aString
	protocol := aString! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 15:50'!
queryString
	"answers the query string or nil if none is present"
	^self detectAttribute: 'query_string'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
remoteAddress
	^ remoteAddress! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:49'!
remoteAddress: aString
	remoteAddress := aString! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
remoteHost
	^ remoteHost! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:49'!
remoteHost: aString
	remoteHost := aString! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:10'!
remoteUser
	"Answer the user name or nil if none is present."
	^self detectAttribute: 'remote_user'! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
requestUri
	^ requestUri! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:49'!
requestUri: aString
	requestUri := aString! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
serverName
	^ serverName! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:49'!
serverName: aString
	serverName := aString! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:48'!
serverPort
	^ serverPort! !

!AJPRequest methodsFor: 'accessing' stamp: 'pmm 7/13/2008 14:49'!
serverPort: anInteger
	serverPort := anInteger! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:09'!
sslCipher
	"Answer the ssl cipher or nil if none is present
	eg. 	'AES128-SHA'"
	^self detectAttribute: 'ssl_cipher'! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:09'!
sslKeySize
	"Answer the ssl key size as Integer or nil if none is present.
	Eg. 128"
	^ self detectAttribute: 'ssl_key_size'! !

!AJPRequest methodsFor: 'accessing-attributes' stamp: 'pmm 10/16/2010 16:09'!
sslSession
	"Answer the ssl session id or nil if none is present
	eg. 	'F315A58CC368A6A3222BC1B1C1956DF86C1B8E973AC8FDBEE0679457172EEADE'"
	^self detectAttribute: 'ssl_session'! !

GRObject subclass: #AJPRequestReader
	instanceVariableNames: 'stream request hasContentLength'
	classVariableNames: 'AttributeName Attributes ContentLength Headers Methods RequestTerminator'
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPRequestReader commentStamp: 'pmm 7/20/2008 10:38' prior: 0!
I build an AJPRequest by reading from an AJPDataStream

Instance Variables
	request:				<AJPDataStream>
	stream:				<AJPRequest>
	hasContentLength:	<Boolean>

request
	- the request that I build by reading from the data stream

stream
	- the ajp data stream from which to read
	
hasContentLength
	- if the content length header has ever been encountered
!

!AJPRequestReader class methodsFor: 'private' stamp: 'pmm 7/20/2008 10:21'!
attributesArray
	^#(
		'context'
		'servlet_path'
		'remote_user'
		'auth_type'
		'query_string'
		'route'
		'ssl_cert'
		'ssl_cipher'
		'ssl_session'
		'req_attribute'
		'ssl_key_size'
		'secret'
		'stored_method'
	)! !

!AJPRequestReader class methodsFor: 'private' stamp: 'pmm 7/13/2008 14:39'!
createMethodsArray
	^#(
		'OPTIONS'
		'GET'
		'HEAD'
		'POST'
		'PUT'
		'DELETE'
		'TRACE'
		'PROPFIND'
		'PROPPATCH'
		'MKCOL'
		'COPY'
		'MOVE'
		'LOCK'
		'UNLOCK'
		'ACL'
		'REPORT'
		'VERSION-CONTROL'
		'CHECKIN'
		'CHECKOUT'
		'UNCHECKOUT'
		'SEARCH'
		'MKWORKSPACE'
		'UPDATE	'
		'LABEL'
		'MERGE'
		'BASELINE_CONTROL'
		'MKACTIVITY'
	)! !

!AJPRequestReader class methodsFor: 'private' stamp: 'pmm 7/13/2008 15:13'!
headersArray
	^#(
		'accept'
		'accept-charset'
		'accept-encoding'
		'accept-language'
		'authorization'
		'connection'
		'content-type'
		'content-length'
		'cookie'
		'cookie2'
		'host'
		'pragma'
		'referer'
		'user-agent'
	)! !

!AJPRequestReader class methodsFor: 'class initialization' stamp: 'pmm 7/20/2008 11:52'!
initialize
	Methods := self createMethodsArray.
	Headers := self headersArray.
	Attributes := self attributesArray.
	ContentLength := 16r08.
	AttributeName := 16r0A.
	RequestTerminator := 16rFF! !

!AJPRequestReader class methodsFor: 'private' stamp: 'pmm 7/13/2008 14:39'!
methodsArray
	^#(
		'OPTIONS'
		'GET'
		'HEAD'
		'POST'
		'PUT'
		'DELETE'
		'TRACE'
		'PROPFIND'
		'PROPPATCH'
		'MKCOL'
		'COPY'
		'MOVE'
		'LOCK'
		'UNLOCK'
		'ACL'
		'REPORT'
		'VERSION-CONTROL'
		'CHECKIN'
		'CHECKOUT'
		'UNCHECKOUT'
		'SEARCH'
		'MKWORKSPACE'
		'UPDATE	'
		'LABEL'
		'MERGE'
		'BASELINE_CONTROL'
		'MKACTIVITY'
	)! !

!AJPRequestReader class methodsFor: 'instance creation' stamp: 'pmm 7/20/2008 15:19'!
on: anAjpDataStream
	^self new
		initializeOn: anAjpDataStream;
		yourself! !

!AJPRequestReader methodsFor: 'reading-headers' stamp: 'pmm 7/20/2008 10:37'!
decodeHeaderName
	"reads a compressed header name"
	| headerCode |
	headerCode := stream readByte.
	
	headerCode = ContentLength
		ifTrue: [ hasContentLength := true ].
	
	^(headerCode > 0 and: [ headerCode <= Headers size ])
		ifTrue: [ Headers at: headerCode ]
		ifFalse: [ 'UNKNOWN' ]! !

!AJPRequestReader methodsFor: 'accessing' stamp: 'pmm 7/20/2008 10:33'!
hasContentLength
	"answers if the content length header has ever been encountered"
	^hasContentLength! !

!AJPRequestReader methodsFor: 'initialize-release' stamp: 'pmm 12/10/2008 07:29'!
initializeOn: anAjpDataStream
	stream := anAjpDataStream.
	request := AJPRequest new.
	hasContentLength := false! !

!AJPRequestReader methodsFor: 'private' stamp: 'pmm 1/10/2009 13:44'!
protocolError: aString
	AJPProtocolError signal: aString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 8/25/2008 18:04'!
read
	"
AJP13_FORWARD_REQUEST :=
    prefix_code      (byte) 0x02 = JK_AJP13_FORWARD_REQUEST
    method           (byte)
    protocol         (string)
    req_uri          (string)
    remote_addr      (string)
    remote_host      (string)
    server_name      (string)
    server_port      (integer)
    is_ssl           (boolean)
    num_headers      (integer)
    request_headers *(req_header_name req_header_value)
    attributes      *(attribut_name attribute_value)
    request_terminator (byte) OxFF"

	self readMethod.
	self readProtocol.
	self readRequestUri.
	self readRemoteAddress.
	self readRemoteHost.
	self readServerName.
	self readServerPort.
	self readIsSsl.
	self readHeaders.
	self readAttributes.
	^request! !

!AJPRequestReader methodsFor: 'reading-attributes' stamp: 'pmm 10/16/2010 16:04'!
readAttribute: type
	"reads a single request attribute from the data stream
	type <Byte> the type of the attribute, see Attributes"
	^(self readAttributeName: type) -> (self readAttributeValue: type)! !

!AJPRequestReader methodsFor: 'reading-attributes' stamp: 'pmm 8/24/2008 16:22'!
readAttributeName: type
	"reads a single request attribute name from the data stream
	type <Byte> the type of the attribute, see Attributes"
	^(type >= 0 and: [ type <= Attributes size ])
		ifTrue: [
			type = AttributeName
				ifTrue: [ stream readString ]
				ifFalse: [ Attributes at: type ] ]
		ifFalse: [ self protocolError: 'unknown attribute type: ' + type printString ]! !

!AJPRequestReader methodsFor: 'reading-attributes' stamp: 'pmm 10/16/2010 16:04'!
readAttributeValue: type
	"Read a single request attribute value from the data stream."
	^type = 16r0B "ssl_key_size"
		ifTrue: [ stream readInteger ]
		ifFalse: [ stream readString ]! !

!AJPRequestReader methodsFor: 'reading-attributes' stamp: 'pmm 12/27/2008 13:57'!
readAttributes
	"reads all the request attributes into an array"
	| attributes |
	attributes := Array streamContents: [ :result |
		| type |
		type := stream readByte.
		[ type = RequestTerminator ] whileFalse: [
			result nextPut: (self readAttribute: type).
			type := stream readByte ] ].
	request attributes: attributes
	! !

!AJPRequestReader methodsFor: 'reading-headers' stamp: 'pmm 7/20/2008 10:32'!
readHeader
	"reads a single http header"
	| leadingByte headerName |
	leadingByte := stream readByte.
	headerName := leadingByte = 16rA0
		"AO means the header name is encoded in a single byte"
		ifTrue: [ self decodeHeaderName ]
		ifFalse: [ stream readStringWithSizeHighByte: leadingByte ].
		
	^headerName -> stream readString! !

!AJPRequestReader methodsFor: 'reading-headers' stamp: 'pmm 7/13/2008 14:58'!
readHeaders
	"reads all http headers"
	| numberOfHeaders headers |
	numberOfHeaders := stream readInteger.
	headers := Array new: numberOfHeaders.
	1 to: numberOfHeaders do: [ :index |
		headers at: index put: self readHeader ].
	request headers: headers.
	! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:53'!
readIsSsl
	request isSsl: stream readBoolean! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/20/2008 10:25'!
readMethod
	"reads and HTTP method from the data stream"
	| methodCode |
	methodCode := stream readByte.
	request method: ((methodCode > 0 and: [ methodCode <= Methods size ])
		ifTrue: [ Methods at: methodCode ]
		ifFalse: [ 'UNKNOWN' ])! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:46'!
readProtocol
	request protocol: stream readString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:50'!
readRemoteAddress
	request remoteAddress: stream readString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:52'!
readRemoteHost
	request remoteHost: stream readString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:50'!
readRequestUri
	request requestUri: stream readString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:52'!
readServerName
	request serverName: stream readString! !

!AJPRequestReader methodsFor: 'reading' stamp: 'pmm 7/13/2008 14:52'!
readServerPort
	request serverPort: stream readInteger! !

GRObject subclass: #AJPResponse
	instanceVariableNames: 'statusCode statusMessage headers body'
	classVariableNames: 'Headers'
	poolDictionaries: ''
	category: 'AJP-Core'!

AJPResponse class
	instanceVariableNames: 'statusCodes'!

AJPResponse class
	instanceVariableNames: 'statusCodes'!

!AJPResponse class methodsFor: 'class initialization' stamp: 'pmm 12/9/2008 19:48'!
initialize
	statusCodes := Dictionary new
		at: 100 put: 'Continue';
		at: 101 put: 'Switching Protocols';
		at: 102 put: 'Processing';
		at: 200 put: 'OK';
		at: 201 put: 'Created';
		at: 202 put: 'Accepted';
		at: 203 put: 'Non-Authoritative Information';
		at: 204 put: 'No Content';
		at: 205 put: 'Reset Content';
		at: 206 put: 'Partial Content';
		at: 207 put: 'Multi-Status';
		at: 300 put: 'Multiple Choices';
		at: 301 put: 'Moved Permanently';
		at: 302 put: 'Found';
		at: 303 put: 'See Other';
		at: 304 put: 'Not Modified';
		at: 305 put: 'Use Proxy';
		at: 307 put: 'Temporary Redirect';
		at: 400 put: 'Bad Request';
		at: 401 put: 'Unauthorized';
		at: 402 put: 'Payment Required';
		at: 403 put: 'Forbidden';
		at: 404 put: 'Not Found';
		at: 405 put: 'Method Not Allowed';
		at: 406 put: 'Not Acceptable';
		at: 407 put: 'Proxy Authentication Required';
		at: 408 put: 'Request Timeout';
		at: 409 put: 'Conflict';
		at: 410 put: 'Gone';
		at: 411 put: 'Length Required';
		at: 412 put: 'Precondition Failed';
		at: 413 put: 'Request Entity Too Large';
		at: 414 put: 'Request-URI Too Long';
		at: 415 put: 'Unsupported Media Type';
		at: 416 put: 'Requested Range Not Satisfiable';
		at: 417 put: 'Expectation Failed';
		at: 418 put: 'I''m a teapot';
		at: 422 put: 'Unprocessable Entity';
		at: 423 put: 'Locked';
		at: 424 put: 'Failed Dependency';
		at: 500 put: 'Internal Server Error';
		at: 501 put: 'Not Implemented';
		at: 502 put: 'Bad Gateway';
		at: 503 put: 'Service Unavailable';
		at: 504 put: 'Gateway Timeout';
		at: 505 put: 'HTTP Version Not Supported';
		at: 507 put: 'Insufficient Storage';
		yourself! !

!AJPResponse class methodsFor: 'instance creation' stamp: 'pmm 12/9/2008 07:33'!
status: anInteger headers: anOrderedCollectionOfAssociations body: aStringOrByteArray
	^self new
		initializeWithStatus: anInteger headers: anOrderedCollectionOfAssociations body: aStringOrByteArray;
		yourself! !

!AJPResponse class methodsFor: 'accessing' stamp: 'pmm 12/9/2008 19:57'!
statusMessageFor: anInteger
	^statusCodes at: anInteger! !

!AJPResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2008 13:34'!
body
	^body! !

!AJPResponse methodsFor: 'accessing' stamp: 'pmm 12/9/2008 07:32'!
headersKeysAndValuesDo: aTwoArgumentBlock
	headers do: [ :each |
		aTwoArgumentBlock
			value: each key
			value: each value ]! !

!AJPResponse methodsFor: 'initialize-release' stamp: 'pmm 12/9/2008 19:59'!
initializeWithStatus: anInteger headers: anOrderedCollectionOfAssociations body: aStringOrByteArray
	statusCode := anInteger.
	statusMessage := self class statusMessageFor: anInteger.
	headers := anOrderedCollectionOfAssociations.
	body := aStringOrByteArray.
	self setDefaultHeaders! !

!AJPResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2008 11:28'!
numberOfHeaders
	^headers size! !

!AJPResponse methodsFor: 'initialize-release' stamp: 'pmm 12/14/2010 07:30'!
setDefaultHeaders
	| contentLengthSet servletEngineSet dateSet |
	contentLengthSet := false.
	headers do: [ :each |
		| key |
		key := each key.
		key = 'Content-Length'
			ifTrue: [ contentLengthSet := true ] ].
	contentLengthSet ifFalse: [
		headers add: 'Content-Length' -> body size printString ]! !

!AJPResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2008 11:27'!
statusCode
	^statusCode! !

!AJPResponse methodsFor: 'accessing' stamp: 'pmm 8/25/2008 11:27'!
statusMessage
	^statusMessage! !

GRObject subclass: #AJPResponseWriter
	instanceVariableNames: 'stream response'
	classVariableNames: 'Headers'
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPResponseWriter class methodsFor: 'private' stamp: 'pmm 8/25/2008 11:46'!
headersDictionary
	^Dictionary new
		at: 'Content-Type' asLowercase put: 16rA001;
		at: 'Content-Language' asLowercase put: 16rA002;
		at: 'Content-Length' asLowercase put: 16rA003;
		at: 'Date' asLowercase put: 16rA004;
		at: 'Last-Modified' asLowercase put: 16rA005;
		at: 'Location' asLowercase put: 16rA006;
		at: 'Set-Cookie' asLowercase put: 16rA007;
		at: 'Set-Cookie2' asLowercase put: 16rA008;
		at: 'Servlet-Engine' asLowercase put: 16rA009; "Apache-Coyote/1.1"
		at: 'Status' asLowercase put: 16rA00A;
		at: 'WWW-Authenticate' asLowercase put: 16rA00B;
		yourself
! !

!AJPResponseWriter class methodsFor: 'class initialization' stamp: 'pmm 8/25/2008 18:18'!
initialize
	Headers := self headersDictionary! !

!AJPResponseWriter class methodsFor: 'instance creation' stamp: 'pmm 8/25/2008 11:12'!
on: anAjpDataStream response: anAjpResponse
	^self new
		initializeOn: anAjpDataStream response: anAjpResponse;
		yourself! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 10/31/2010 10:58'!
bodyChunkAt: index
	| body bodySize maximumBodySize chunkSize start result |
	body := response body.
	bodySize := body size.
	maximumBodySize := self maximumBodySize.

	chunkSize := index * maximumBodySize > bodySize
		ifTrue: [ bodySize \\ maximumBodySize ]
		ifFalse: [ maximumBodySize ].
		
	start := ((index - 1) * maximumBodySize) + 1.
	^ AJPBodyChunk start: start size: chunkSize body: body! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 8/25/2008 11:23'!
endResponseCode
	^5! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 11/23/2008 10:05'!
headersSize
	| sum |
	sum := 0.
	response headersKeysAndValuesDo: [ :key :value |
		sum := sum + ((self isCompressed: key)
			ifTrue: [ 2 ]
			ifFalse: [ key size + 3 ]).
		sum := sum + value size + 3 ].
	^sum! !

!AJPResponseWriter methodsFor: 'initialize-release' stamp: 'pmm 8/25/2008 11:12'!
initializeOn: anAjpDataStream response: anAjpResponse
	stream := anAjpDataStream.
	response := anAjpResponse! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 11/23/2008 09:57'!
isCompressed: aString
	"checks whether the given string is a compressed header"
	^Headers includesKey: aString asLowercase! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 8/25/2008 12:09'!
maximumBodySize
	^16r2000 "maximum packet size"
		- 2 "prefix"
		- 1 "type"
		- 2 "size"! !

!AJPResponseWriter methodsFor: 'accessing' stamp: 'pmm 8/25/2008 14:13'!
numberOfBodyChunks
	| body bodySize modulo |
	body := response body.
	body isEmptyOrNil
		ifTrue: [ ^0 ].
		
	bodySize := body size.
	modulo := bodySize \\ self maximumBodySize.
	^modulo isZero
		ifTrue: [ bodySize // self maximumBodySize ]
		ifFalse: [ (bodySize // self maximumBodySize) + 1 ]! !

!AJPResponseWriter methodsFor: 'writing' stamp: 'pmm 10/31/2010 11:17'!
sendBodyChunk: index
	| chunk |
	chunk := self bodyChunkAt: index.
	stream writeInteger: 1 + 2 + chunk size + 1. "packet size (prefix + chunk size + chunk + terminator)"
	stream writeByte: self sendBodyChunkCode.
	stream writeInteger: chunk size.
	stream writeChunk: chunk size from: chunk body startingAt: chunk start.
	stream writeByte: 0 "terminator"! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 8/25/2008 11:23'!
sendBodyChunkCode
	^3! !

!AJPResponseWriter methodsFor: 'writing' stamp: 'pmm 11/23/2008 11:20'!
sendEndResponse: reuse
	"sends the server a package that this response is complete
	reuse: aBoolean, whether the connection should be reused"
	stream writeInteger: 2. "size"
	stream writeByte: self endResponseCode.
	stream writeBoolean: reuse! !

!AJPResponseWriter methodsFor: 'writing-private' stamp: 'pmm 8/25/2008 11:48'!
sendHeaderName: aString
	| compressed |
	compressed := false.
	Headers at: aString asLowercase ifPresent: [ :code |
		stream writeInteger: code.
		compressed := true ].
	compressed ifFalse: [
		stream writeString: aString]! !

!AJPResponseWriter methodsFor: 'writing-private' stamp: 'pmm 8/25/2008 11:48'!
sendHeaderValue: aString
	stream writeString: aString! !

!AJPResponseWriter methodsFor: 'writing' stamp: 'pmm 11/23/2008 10:01'!
sendHeaders
	stream writeInteger: self sendHeadersSize.
	stream writeByte: self sendHeadersCode.
	stream writeInteger: response statusCode.
	stream writeString: response statusMessage.
	stream writeInteger: response numberOfHeaders.
	response headersKeysAndValuesDo: [ :key :value |
		self sendHeaderName: key.
		self sendHeaderValue: value ]! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 8/25/2008 11:22'!
sendHeadersCode
	^4! !

!AJPResponseWriter methodsFor: 'private' stamp: 'pmm 11/23/2008 10:05'!
sendHeadersSize
	^1 + "prefix code"
		2 + "status code"
		response statusMessage size + 3 +
		2 + "num headers"
		self headersSize! !

GRObject subclass: #AJPServer
	instanceVariableNames: 'adapter shouldStop mutex workers status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPServer methodsFor: 'private' stamp: 'pmm 1/6/2009 06:48'!
backlogSize
	^50! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:48'!
beStarted
	status := #started! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:49'!
beStarting
	status := #starting! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:48'!
beStopped
	status := #stopped! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:50'!
beStopping
	status := #stopping! !

!AJPServer methodsFor: 'initialize-release' stamp: 'pmm 8/29/2009 09:50'!
initialize
	super initialize.
	shouldStop := false.
	workers := IdentitySet new.
	mutex := Semaphore forMutualExclusion.
	self beStopped! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:48'!
isStarted
	^status == #started! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:48'!
isStarting
	^status == #starting! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:48'!
isStopped
	^status == #stopped! !

!AJPServer methodsFor: 'testing' stamp: 'pmm 8/29/2009 09:50'!
isStopping
	^status == #stopping! !

!AJPServer methodsFor: 'private' stamp: 'pmm 12/28/2008 16:16'!
shouldStop
	^mutex critical: [ shouldStop ]! !

!AJPServer methodsFor: 'actions' stamp: 'pmm 8/29/2009 09:50'!
stop
	self beStopping.
	mutex critical: [ shouldStop := true ].
	mutex critical: [
		workers do: [ :each |
			each destroy ].
		workers := IdentitySet new ].
	self beStopped! !

GRError subclass: #AJPLoopTerminatedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPLoopTerminatedError commentStamp: 'pmm 8/29/2009 10:09' prior: 0!
An AJPLoopTerminatedError signals that an AJP loop should exit because the server is stopped or the socket has been closed.!

GRError subclass: #AJPProtocolError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AJP-Core'!

!AJPProtocolError commentStamp: 'pmm 7/13/2008 10:09' prior: 0!
An AJPProtocolError signals an exception in the AJP Protocol somewhere.
!
AJPRequestReader initialize!
AJPResponse initialize!
AJPResponseWriter initialize!
